﻿Imports System.Data.Entity
Imports CustomerManagement.Core.DomainEvents
Imports CustomerManagement.Core.Model
Imports CustomerManagement.Core.Services
Imports CustomerManagement.Infrastructure.ApplicationEvents
Imports Shareds

Namespace CustomerManagement.Infrastructure
  Public Class CustomerAggregateRepository
	Public Function PersistNewCustomer(ByVal customer As Customer) As Boolean
	  Using context = New CustomerAggregateContext()
		context.Customers.Add(customer)
		Dim response As Integer = context.SaveChanges()
		If response > 0 Then
		  PublishCustomerPersistedEvent(customer, True)
		  Return True
		End If
		Return False
	  End Using
	End Function

	Public Function PersistChangeToCustomer(ByVal customer As Customer) As Boolean
	  Using context = New CustomerAggregateContext()
		context.Customers.Attach(customer)
		context.Entry(customer).State = EntityState.Modified

		Dim response As Integer = context.SaveChanges()
		If response > 0 Then
		  If customer.Events.OfType(Of CustomerNameFixedEvent)().Any() Then
			PublishCustomerPersistedEvent(customer, False)
		  End If
		  Return True
		End If
		Return False
	  End Using
	End Function

	Private Sub PublishCustomerPersistedEvent(ByVal customer As Customer, ByVal isNew As Boolean)
	  Dim dto As CustomerDto = CustomerDto.Create(customer.Id, customer.Name)
	  DomainEvents.Raise(New CustomerUpdatedEvent(dto, isNew))
	End Sub
  End Class
End Namespace