﻿Imports System.Data.Entity
Imports System.Data.Entity.Infrastructure
Imports CustomerManagement.Core.Model
Imports CustomerManagement.Infrastructure
Imports CustomerManagementTests.DependencyResolution
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports StructureMapContainer

Namespace CustomerManagementTests
  <TestClass>
  Public Class PublishToRabbitMqTests
	Public Sub New()
	  IocContainer.Container = IoC.Container
	  Database.SetInitializer(New NullDatabaseInitializer(Of CustomerAggregateContext)())
	End Sub

	<TestMethod>
	Public Sub CanSendMessageToQueueOnSuccessfulCustomerInsert()
	  Dim customer As Customer = Customer.Create("Julie Lerman", "Friend Referral")
	  Dim repo = New CustomerAggregateRepository()
	  repo.PersistNewCustomer(customer)

	  Assert.Inconclusive("Check status of RabbitMQ Manager for this message")
	End Sub

	<TestMethod>
	Public Sub CanSendMessageToQueueOnSuccessfulCustomerNameUpdate()
	  Dim customer As Customer = Customer.Create("Spamson Lerman", "Friend Referral")
	  Dim repo = New CustomerAggregateRepository()
	  repo.PersistNewCustomer(customer)
	  customer.FixName("Sampson Lerman")

	  repo.PersistChangeToCustomer(customer)

	  Assert.Inconclusive("Check status of RabbitMQ Manager for a create message and an update message")
	End Sub

	<TestMethod>
	Public Sub WillNotSendMessageToQueueOnSuccessfulCustomerAddressUpdate()
	  Dim customer As Customer = Customer.Create("George Jetson", "Friend Referral")
	  Dim repo = New CustomerAggregateRepository()
	  repo.PersistNewCustomer(customer)
	  customer.CreateNewBillingAddress("123 SkyPad Apartments", "", "Orbit City", "Orbit", "n/a", "")
	  repo.PersistChangeToCustomer(customer)

	  Assert.Inconclusive("Check status of RabbitMQ Manager for a create message, " & ControlChars.CrLf & "                          but no update message")
	End Sub

	''' <summary>
	''' this one was just for curiousity 
	''' if the savechanges throws then the message won't get
	''' sent. I just wanted to make sure my 
	''' if statements were working.
	''' </summary>
	<TestMethod, ExpectedException(GetType(DbUpdateConcurrencyException))>
	Public Sub WillNotSendMessageToQueueOnFailedCustomerUpdate()
	  Dim customer As Customer = Customer.Create("Unpersisted Customer", "Friend Referral")
	  Dim repo = New CustomerAggregateRepository()
	  customer.FixName("WasNeverPersisted Customer")

	  'note, did not persist the new customer, so update will fail
	  'and message will not get sent
	  repo.PersistChangeToCustomer(customer)

	End Sub
  End Class
End Namespace