﻿'From Pluralsight Domain-Driven Design Fundamentals Course (bit.ly/PS-DDD)


Namespace Shareds
    Public Class DateTimeRange
        Inherits ValueObject(Of DateTimeRange)
        Private privateStart As Date
        Public Property Start() As Date
            Get
                Return privateStart
            End Get
            Private Set(ByVal value As Date)
                privateStart = value
            End Set
        End Property
        Private privateEnd As Date
        Public Property [End]() As Date
            Get
                Return privateEnd
            End Get
            Private Set(ByVal value As Date)
                privateEnd = value
            End Set
        End Property

        Public Sub New(ByVal start As Date, ByVal [end] As Date)
            Guard.ForPrecedesDate(start, [end], "start")
            Me.Start = start
            Me.End = [end]
        End Sub

        Public Sub New(ByVal start As Date, ByVal duration As TimeSpan)
            Me.New(start, start.Add(duration))
        End Sub
        Protected Sub New()
        End Sub

        Public Function DurationInMinutes() As Integer
            Return ([End].Subtract(Start)).Minutes
        End Function

        'INSTANT VB NOTE: The parameter newEnd was renamed since Visual Basic will not allow parameters with the same name as their enclosing function or property:
        Public Function NewEnd(ByVal newEnd_Renamed As Date) As DateTimeRange
            Return New DateTimeRange(Me.Start, newEnd_Renamed)
        End Function
        'INSTANT VB NOTE: The parameter newDuration was renamed since Visual Basic will not allow parameters with the same name as their enclosing function or property:
        Public Function NewDuration(ByVal newDuration_Renamed As TimeSpan) As DateTimeRange
            Return New DateTimeRange(Me.Start, newDuration_Renamed)
        End Function
        'INSTANT VB NOTE: The parameter newStart was renamed since Visual Basic will not allow parameters with the same name as their enclosing function or property:
        Public Function NewStart(ByVal newStart_Renamed As Date) As DateTimeRange
            Return New DateTimeRange(newStart_Renamed, Me.End)
        End Function

        Public Shared Function CreateOneDayRange(ByVal day As Date) As DateTimeRange
            Return New DateTimeRange(day, day.AddDays(1))
        End Function

        Public Shared Function CreateOneWeekRange(ByVal startDay As Date) As DateTimeRange
            Return New DateTimeRange(startDay, startDay.AddDays(7))
        End Function

        Public Function Overlaps(ByVal dateTimeRange As DateTimeRange) As Boolean
            Return Me.Start < dateTimeRange.End AndAlso Me.End > dateTimeRange.Start
        End Function
    End Class
End Namespace
