﻿Imports System.Text
Imports Newtonsoft.Json
Imports RabbitMQ.Client
Imports RabbitMQ.Client.Events
Imports RabbitMQ.Client.MessagePatterns
Imports Shareds

Namespace OrderManagement.DataMirroring
  ''' <summary>
  '''   Class to encapsulate recieving messages from RabbitMQ
  '''   Derived from sample code on Pluralsight "RabbitMQ for .NET Developers" course
  ''' </summary>
  Public Class RabbitListener
	  Implements IDisposable
	Public Delegate Sub OnReceiveMessage(ByVal message As String)

	Private Const QueueName As String = "MyMSDNSampleQueue"
	Private Const IsDurable As Boolean = True


	Private ReadOnly _channel As IModel
	Private _exchangeName As String
	Private _subscription As Subscription


	Public Sub New()
	  Dim factory = New ConnectionFactory()
	  Dim conn As IConnection = factory.CreateConnection() 'uses defaults
	  _channel = conn.CreateModel()
	  _exchangeName = "CustomerUpdate"
	  _channel.ExchangeDeclare(_exchangeName, ExchangeType.Direct)
	  _channel.QueueDeclare(QueueName, False, False, False, Nothing)
	  _channel.QueueBind(QueueName, _exchangeName, "", Nothing)
	  Dim props As IBasicProperties = _channel.CreateBasicProperties()

	  _channel.BasicQos(0, 1, False)
	End Sub

	Public Property Enabled() As Boolean

	Public Sub Dispose() Implements IDisposable.Dispose
	  If _channel IsNot Nothing Then
		_channel.Dispose()
	  End If
	  GC.SuppressFinalize(Me)
	End Sub

	Public Sub Start()
	  _subscription = New Subscription(_channel, QueueName, False)

	  Dim consumer = New ConsumeDelegate(AddressOf Poll)
	  consumer.Invoke()
	End Sub

	Private Sub Poll()
	  Do While Enabled
		Console.WriteLine("Waiting for messages!")
		'Get next message
		Dim deliveryArgs As BasicDeliverEventArgs = _subscription.Next()
		'Deserialize message
		Dim message As String = Encoding.Default.GetString(deliveryArgs.Body)

		'Handle Message
		Console.WriteLine("Message Recieved - {0}", message)
		Dim customerUpdatedEvent = JsonConvert.DeserializeObject(Of CustomerUpdatedEvent)(message)
		DomainEvents.Raise(customerUpdatedEvent)
		_subscription.Ack()
	  Loop
	End Sub

	Private Delegate Sub ConsumeDelegate()
  End Class
End Namespace